@@ -26,31 +26,50 @@
  */
 
 #include <util.h>
-/*@unused@*/ RCSID("$Id: elf-x86-x86.c 1976 2007-09-21 16:48:25Z peter $");
+/*@unused@*/ RCSID("$Id: elf-x86-x86.c 2210 2009-07-22 05:51:35Z peter $");
 
 #include <libyasm.h>
 #define YASM_OBJFMT_ELF_INTERNAL
 #include "elf.h"
 #include "elf-machine.h"
 
-enum ssym_index {
-    SSYM_GOTPC = 0,
-    SSYM_GOTOFF,
-    SSYM_GOT,
-    SSYM_PLT
+static const elf_machine_ssym elf_x86_x86_ssyms[] = {
+    {"plt",         ELF_SSYM_SYM_RELATIVE,  R_386_PLT32,        32},
+    {"gotoff",      0,                      R_386_GOTOFF,       32},
+    /* special one for NASM */
+    {"gotpc",       ELF_SSYM_CURPOS_ADJUST, R_386_GOTPC,        32},
+    {"tlsgd",       ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_386_TLS_GD,       32},
+    {"tlsldm",      ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_386_TLS_LDM,      32},
+    {"gottpoff",    ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_386_TLS_IE_32,    32},
+    {"tpoff",       ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_386_TLS_LE_32,    32},
+    {"ntpoff",      ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_386_TLS_LE,       32},
+    {"dtpoff",      ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_386_TLS_LDO_32,   32},
+    {"gotntpoff",   ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_386_TLS_GOTIE,    32},
+    {"indntpoff",   ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_386_TLS_IE,       32},
+    {"got",         ELF_SSYM_SYM_RELATIVE,  R_386_GOT32,        32},
+    {"tlsdesc",     ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_386_TLS_GOTDESC,  32},
+    {"tlscall",     ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_386_TLS_DESC_CALL,    32}
 };
 
 static int
-elf_x86_x86_accepts_reloc(size_t val, yasm_symrec *wrt, yasm_symrec **ssyms)
+elf_x86_x86_accepts_reloc(size_t val, yasm_symrec *wrt)
 {
     if (wrt) {
-        if ((wrt == ssyms[SSYM_GOTPC] && val == 32)
-            || (wrt == ssyms[SSYM_GOTOFF] && val == 32)
-            || (wrt == ssyms[SSYM_GOT] && val == 32)
-            || (wrt == ssyms[SSYM_PLT] && val == 32))
-            return 1;
-        else
+        const elf_machine_ssym *ssym = (elf_machine_ssym *)
+            yasm_symrec_get_data(wrt, &elf_ssym_symrec_data);
+        if (!ssym || val != ssym->size)
             return 0;
+        return 1;
     }
     return (val&(val-1)) ? 0 : ((val & (8|16|32)) != 0);
 }
@@ -117,26 +136,50 @@ elf_x86_x86_write_secthead_rel(unsigned char *bufp,
 }
 
 static void
-elf_x86_x86_handle_reloc_addend(yasm_intnum *intn, elf_reloc_entry *reloc)
+elf_x86_x86_handle_reloc_addend(yasm_intnum *intn,
+                                elf_reloc_entry *reloc,
+                                unsigned long offset)
 {
+    int is_GOT = reloc->is_GOT_sym;
+
+    if (reloc->wrt) {
+        const elf_machine_ssym *ssym = (elf_machine_ssym *)
+            yasm_symrec_get_data(reloc->wrt, &elf_ssym_symrec_data);
+        if (!ssym || reloc->valsize != ssym->size)
+            yasm_internal_error(N_("Unsupported WRT"));
+        if (ssym->reloc == R_386_GOTPC)
+            is_GOT = 1;
+    }
+
+    if (is_GOT && reloc->valsize == 32 && offset != 0)
+    {
+        yasm_intnum *off_intn = yasm_intnum_create_uint(offset);
+        yasm_intnum_calc(intn, YASM_EXPR_ADD, off_intn);
+        yasm_intnum_destroy(off_intn);
+    }
     return; /* .rel: Leave addend in intn */
 }
 
 static unsigned int
-elf_x86_x86_map_reloc_info_to_type(elf_reloc_entry *reloc,
-                                   yasm_symrec **ssyms)
+elf_x86_x86_map_reloc_info_to_type(elf_reloc_entry *reloc)
 {
     if (reloc->wrt) {
-        if (reloc->wrt == ssyms[SSYM_GOTPC] && reloc->valsize == 32)
-            return (unsigned char) R_386_GOTPC;
-        else if (reloc->wrt == ssyms[SSYM_GOTOFF] && reloc->valsize == 32)
-            return (unsigned char) R_386_GOTOFF;
-        else if (reloc->wrt == ssyms[SSYM_GOT] && reloc->valsize == 32)
-            return (unsigned char) R_386_GOT32;
-        else if (reloc->wrt == ssyms[SSYM_PLT] && reloc->valsize == 32)
-            return (unsigned char) R_386_PLT32;
-        else
+        const elf_machine_ssym *ssym = (elf_machine_ssym *)
+            yasm_symrec_get_data(reloc->wrt, &elf_ssym_symrec_data);
+        if (!ssym || reloc->valsize != ssym->size)
             yasm_internal_error(N_("Unsupported WRT"));
+
+        /* Force TLS type; this is required by the linker. */
+        if (ssym->sym_rel & ELF_SSYM_THREAD_LOCAL) {
+            elf_symtab_entry *esym;
+
+            esym = yasm_symrec_get_data(reloc->reloc.sym, &elf_symrec_data);
+            if (esym)
+                esym->type = STT_TLS;
+        }
+        return (unsigned char) ssym->reloc;
+    } else if (reloc->is_GOT_sym && reloc->valsize == 32) {
+        return (unsigned char) R_386_GOTPC;
     } else if (reloc->rtype_rel) {
         switch (reloc->valsize) {
             case 8: return (unsigned char) R_386_PC8;
@@ -193,13 +236,6 @@ elf_x86_x86_write_proghead(unsigned char **bufpp,
     *bufpp = bufp;
 }
 
-static elf_machine_ssym elf_x86_x86_ssyms[] = {
-    {"..gotpc", ELF_SSYM_CURPOS_ADJUST},
-    {"..gotoff", 0},
-    {"..got", ELF_SSYM_SYM_RELATIVE},
-    {"..plt", 0}
-};
-
 const elf_machine_handler
 elf_machine_handler_x86_x86 = {
     "x86", "x86", ".rel",
